# (c) Copyright 2011 Cloudera, Inc. All rights reserved.

"""
Module for communicating with the firehose.

This is copied over from the CMON agent. We need to merge them back
at some point.
"""

import os
import logging
import time
import avro.ipc
import avro.protocol

from cmf.monitor.base import DebugStatusReport
from cmf.throttling_logger import ThrottlingLogger

LOG = logging.getLogger(__name__)

_CONN_FAILED_LOG_DELAY = 10 * 60
THROTTLED_LOG = ThrottlingLogger(LOG, _CONN_FAILED_LOG_DELAY)

FIREHOSE_MESSAGE_PROTOCOL = avro.protocol.parse(
  file(os.path.join(os.path.dirname(__file__), "cmon_agent.avpr")).read())

class Firehose(object):
  """This class is used for communication with a firehose"""

  def __init__(self, name, roletype, address, port):
    self.name = name
    self.roletype = roletype
    self._socket = None
    self._connected = False
    self._address = address
    self._port = port
    self._last_conn_error = None
    self._next_conn_failed_message = time.time()
    self._bytes_sent = 0
    self._requestor = None

  def set_connection_info(self, address, port):
    if self._address == address and self._port == port:
      return
    if address is not None and port is not None:
      LOG.info("Updating firehose connection to %s:%s:" % (address, port))
    else:
      LOG.info("Updating firehose connection to be unconfigured.")
    if self._connected:
      self._socket.close()
    self._connected = False
    self._address = address
    self._port = port

  def configured(self):
    return (self._address is not None and self._port is not None)

  def send(self, messages):
    """Sends messages to the firehose. Returns True on success."""
    try:
      if self._requestor is None:
        transceiver = avro.ipc.HTTPTransceiver(self._address,
                                               self._port)
        self._requestor = avro.ipc.Requestor(FIREHOSE_MESSAGE_PROTOCOL,
                                             transceiver)

      self._requestor.request('sendAgentMessages', dict(messages=messages))
      return True
    except Exception:
      THROTTLED_LOG.exception("Error sending messages to firehose: " + self.name)
      self._requestor = None
      return False

  def get_debug_status_report(self):
    return DebugStatusReport(config_items=[
        ('address', self._address),
        ('port', self._port),
        ('connected', self._connected),
        ('last connection error', self._last_conn_error ),
        ('next connection failed log message', 
          time.ctime(self._next_conn_failed_message)),
        ('bytes sent', self._bytes_sent)])

  def __str__(self):
    return "Firehose(%s, %s, %s, %s)" % (self.name,
                                         self.roletype,
                                         self._address,
                                         self._port)
